#include        <windows.h>
#include        <gl\gl.h>
#include        <gl\glu.h>
#include 		 <math.h>
#include	       <stdlib.h>

#define 			 NUMPOINTS  20
#define			 NUMSTARS	200
#define			 R		      (8.0 + light * 5.0)
#define			 H			   (10.0 - light * 5.0)

const GLfloat   K = 360.0 / NUMPOINTS * M_PI / 180.0;

LONG WINAPI wndProc ( HWND, UINT, WPARAM, LPARAM );
void        setDCPixelFormat ( HDC );
void        initializeRC ();
void        drawBox ( GLfloat, GLfloat, GLfloat, GLfloat, GLfloat, GLfloat );
void        drawScene ( HDC, int, int, int );
UINT 			timer;
int			dir = 0;
GLfloat light = 0.0;
int	      stars_x[NUMSTARS];
int	      stars_y[NUMSTARS];
int	      stars_z[NUMSTARS];


int     WINAPI WinMain ( HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR cmdLine, int cmdShow )
{
        static char appName [] = "Cylinder - by HackMaster";
        WNDCLASS    wc;
        HWND        hWnd;
        MSG         msg;

        for (int i = 0; i < NUMSTARS; i++)
        {
          stars_x[i] = random(100) - 50;
          stars_y[i] = random(100) - 50;
          stars_z[i] = -random(100);
        }

        wc.style         = CS_HREDRAW | CS_VREDRAW;
        wc.lpfnWndProc   = (WNDPROC) wndProc;
        wc.cbClsExtra    = 0;
        wc.cbWndExtra    = 0;
        wc.hInstance     = hInstance;
        wc.hIcon         = LoadIcon ( NULL, IDI_APPLICATION );
        wc.hCursor       = LoadCursor ( NULL, IDC_ARROW );
        wc.hbrBackground = (HBRUSH) (COLOR_WINDOW + 1);
        wc.lpszMenuName  = NULL;
        wc.lpszClassName = appName;

        RegisterClass ( &wc );

        hWnd = CreateWindow ( appName, appName,
                              WS_OVERLAPPEDWINDOW | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,
                              CW_USEDEFAULT, CW_USEDEFAULT, 320,
                              240, HWND_DESKTOP, NULL, hInstance, NULL );

        ShowWindow ( hWnd, cmdShow );
        UpdateWindow ( hWnd );

        while ( GetMessage ( &msg, NULL, 0, 0 ) )
        {
                TranslateMessage ( &msg );
                DispatchMessage  ( &msg );
        }

        return msg.wParam;
}

LONG WINAPI wndProc ( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
        static HDC   hdc;
        static HGLRC hrc;
        PAINTSTRUCT  ps;
        GLdouble     aspect;
        static int   angle_x = 0, angle_y = 0, angle_z = 0;
		  GLsizei      width, height;

        switch ( msg )
        {
                case WM_CREATE:
                	      randomize();
                        hdc = GetDC ( hWnd );
                        setDCPixelFormat ( hdc );
                        hrc = wglCreateContext ( hdc );
                        ReleaseDC ( hWnd, hdc );
                        timer = SetTimer(hWnd, 1, 50, NULL);
                        return 0;

                case WM_SIZE:
                        hdc = GetDC ( hWnd );
                        wglMakeCurrent ( hdc, hrc );

                        width  = (GLsizei) LOWORD ( lParam );
                        height = (GLsizei) HIWORD ( lParam );

                        if (!width || !height) return 0;

                        aspect = (GLdouble) width / (GLdouble) height;

                        glMatrixMode ( GL_PROJECTION );
                        glLoadIdentity ();
                        gluPerspective ( 30.0, aspect, 1.0, 1000.0 );

                        glViewport ( 0, 0, width, height );

                        wglMakeCurrent ( NULL, NULL );
                        ReleaseDC ( hWnd, hdc );

                        return 0;

                case WM_PAINT:
                        hdc = BeginPaint ( hWnd, &ps );
                        wglMakeCurrent ( hdc, hrc );
                        drawScene  ( hdc, angle_x, angle_y, angle_z );
                        wglMakeCurrent ( NULL, NULL );
                        EndPaint   ( hWnd, &ps );

                        return 0;

                case WM_DESTROY:
                        wglMakeCurrent ( NULL, NULL );
                        wglDeleteContext ( hrc );
                        KillTimer(hWnd, timer);
                        PostQuitMessage ( 0 );

                        return 0;
                case WM_TIMER:
                        angle_x += 2;
                        angle_y += 2;
                        angle_x %= 360;
                        angle_y %= 360;
                        if (dir)
                        	light -= 0.025;
                        else
                        	light += 0.025;
                        if (light > 1)
                        {
                          dir = 1;
                          light = 1;
                        }
                        if (light < 0)
                        {
                          dir = 0;
                          light = 0;
                        }
                        InvalidateRect(hWnd, NULL, FALSE);
                        return 0;

        }

        return DefWindowProc ( hWnd, msg, wParam, lParam );
}

void    setDCPixelFormat ( HDC hdc )
{
        static PIXELFORMATDESCRIPTOR pfd =
        {
                sizeof ( PIXELFORMATDESCRIPTOR ),       // sizeof this structure
                1,                                      // version number
                PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,// flags
                PFD_TYPE_RGBA,                          // RGBA pixel values
                16,                                     // 24-bit color
                0, 0, 0, 0, 0, 0,                       // don't care about these
                0, 0,                                   // no alpha buffer
                0, 0, 0, 0, 0,                          // no accumulation buffer
                16,                                     // 32-bit depth buffer
                0,                                      // no stencil buffer
                0,                                      // no auxiliary buffers
                PFD_MAIN_PLANE,                         // layer type
                0,                                      // reserved (must be 0)
                0, 0, 0                                 // no layer masks
        };
        int     pixelFormat;

        pixelFormat = ChoosePixelFormat ( hdc, &pfd );
        SetPixelFormat ( hdc, pixelFormat, &pfd );
        DescribePixelFormat ( hdc, pixelFormat,
                              sizeof (PIXELFORMATDESCRIPTOR), &pfd );
}

void    initializeRC ()
{
        GLfloat lightAmbient  [] = { 0.07, 0.07, 0.07, 1.0 };
        GLfloat lightDiffuse  [] = { 0.7, 0.7, 0.7, 1.0 };
        GLfloat lightSpecular [] = { 0.3, 0.3, 0.3, 1.0 };

        glFrontFace  ( GL_CW );
        glCullFace   ( GL_BACK );
        glEnable     ( GL_CULL_FACE );

        glDepthFunc  ( GL_LEQUAL );
        glEnable     ( GL_DEPTH_TEST );
        glShadeModel ( GL_SMOOTH );
        glClearColor ( 0.0, 0.0, 0.25, 0.0 );

        glLightfv ( GL_LIGHT0, GL_AMBIENT, lightAmbient );
        glLightfv ( GL_LIGHT0, GL_DIFFUSE, lightDiffuse );
        glLightfv ( GL_LIGHT0, GL_SPECULAR, lightSpecular );
        glEnable  ( GL_LIGHTING );
        glEnable  ( GL_LIGHT0 );
}

void    drawScene ( HDC hDC, int angle_x, int angle_y, int angle_z )
{
        GLfloat blue [] = { light, light, 1.0 - light, 1.0 };
        GLfloat yellow [] = { 1.0 - light, 1.0 - light, light };
        GLfloat x, y;

        initializeRC();
        glClear ( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

        glMatrixMode ( GL_MODELVIEW );
        glLoadIdentity ();

        glTranslatef(0.0, 0.0, -64);

        glPointSize(1);
        glDisable( GL_LIGHTING );
        glBegin(GL_POINTS);
        for (int i = 0; i < NUMSTARS; i++)
        {
          glColor4f(1.0, 1.0, 1.0, 1.0);
          glVertex3s(stars_x[i], stars_y[i], stars_z[i]);
          stars_z[i] += 3;
          if (stars_z[i] >= 0)
            stars_z[i] = -100;
        }
        glEnd();
	     glEnable(GL_LIGHTING);

        glRotatef((GLfloat)angle_x, 1.0, 0.0, 0.0);
        glRotatef((GLfloat)angle_y, 0.0, 1.0, 0.0);
        glRotatef((GLfloat)angle_z, 0.0, 0.0, 1.0);

		  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, yellow);

        glBegin(GL_QUAD_STRIP);
          glNormal3f(1.0, 0.0, 0.0);
          glVertex3f(R, 0.0, -H);
          glVertex3f(R, 0.0, H);
          for (int i = 1; i <= NUMPOINTS; i++)
          {
            x = cos(i * K) * R;
            y = sin(i * K) * R;
            glNormal3f(x / R, y / R, 0);
            glVertex3f(x, y, -H);
            glVertex3f(x, y, H);
          }
        glEnd();

		  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, blue);
        glNormal3f(0.0, 0.0, 1);
        glVertex3f(0.0, 0.0, H);

        glBegin(GL_TRIANGLE_FAN);
          for (int i = NUMPOINTS - 1; i >= 0 ; i--)
          {
            x = cos(i * K) * R;
            y = sin(i * K) * R;
            glNormal3f(0.0, 0.0, 1);
            glVertex3f(x, y, H);
          }
        glEnd();

        glNormal3f(0.0, 0.0, -1);
        glVertex3f(0.0, 0.0, -H);

        glBegin(GL_TRIANGLE_FAN);
          for (int i = 0; i < NUMPOINTS; i++)
          {
            x = cos(i * K) * R;
            y = sin(i * K) * R;
            glNormal3f(0.0, 0.0, -1);
            glVertex3f(x, y, -H);
          }
        glEnd();

        glFlush ();                     // render the scene to pixel buffer
        SwapBuffers(hDC);
}

